@@ -21,6 +21,14 @@ impl Buffer {
         }
     }
 
+    pub fn channel(&self) -> Option<&str> {
+        match self {
+            Buffer::Server(_) => None,
+            Buffer::Channel(_, channel) => Some(channel),
+            Buffer::Query(_, _) => None,
+        }
+    }
+
     pub fn target(&self) -> Option<String> {
         match self {
             Buffer::Server(_) => None,
@@ -37,6 +45,7 @@ impl Buffer {
             Self::Channel(_, channel) => message::Target::Channel {
                 channel,
                 source: message::Source::Server(source),
+                prefix: None,
             },
             Self::Query(_, nick) => message::Target::Query {
                 nick,
@@ -60,25 +69,74 @@ impl From<config::Buffer> for Settings {
 }
 
 #[derive(Debug, Clone, Copy, Default, Deserialize)]
-pub enum InputVisibility {
+pub struct TextInput {
+    #[serde(default)]
+    pub visibility: TextInputVisibility,
+    #[serde(default)]
+    pub auto_format: AutoFormat,
+}
+
+#[derive(Debug, Clone, Copy, Default, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+pub enum TextInputVisibility {
     Focused,
     #[default]
     Always,
 }
 
+#[derive(Debug, Clone, Copy, Default, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+pub enum AutoFormat {
+    #[default]
+    Disabled,
+    Markdown,
+    All,
+}
+
 #[derive(Debug, Clone, Deserialize)]
 pub struct Timestamp {
+    #[serde(default = "default_timestamp")]
     pub format: String,
     #[serde(default)]
     pub brackets: Brackets,
 }
 
-#[derive(Debug, Clone, Default, Deserialize)]
+impl Default for Timestamp {
+    fn default() -> Self {
+        Self {
+            format: default_timestamp(),
+            brackets: Default::default(),
+        }
+    }
+}
+
+#[derive(Debug, Clone, Deserialize)]
 pub struct Nickname {
     #[serde(default)]
     pub color: Color,
     #[serde(default)]
     pub brackets: Brackets,
+    #[serde(default)]
+    pub alignment: Alignment,
+    #[serde(default = "default_bool_true")]
+    pub show_access_levels: bool,
+}
+
+impl Default for Nickname {
+    fn default() -> Self {
+        Self {
+            color: Default::default(),
+            brackets: Default::default(),
+            alignment: Default::default(),
+            show_access_levels: default_bool_true(),
+        }
+    }
+}
+
+#[derive(Debug, Clone, Default, Deserialize)]
+pub struct StatusMessagePrefix {
+    #[serde(default)]
+    pub brackets: Brackets,
 }
 
 #[derive(Debug, Clone, Default, Deserialize)]
@@ -94,12 +152,27 @@ impl Brackets {
 }
 
 #[derive(Debug, Clone, Copy, Default, Deserialize)]
+#[serde(rename_all = "kebab-case")]
 pub enum Color {
     Solid,
     #[default]
     Unique,
 }
 
+#[derive(Debug, Clone, Copy, PartialEq, Eq, Default, Deserialize)]
+#[serde(rename_all = "kebab-case")]
+pub enum Alignment {
+    #[default]
+    Left,
+    Right,
+}
+
+impl Alignment {
+    pub fn is_right(&self) -> bool {
+        matches!(self, Self::Right)
+    }
+}
+
 #[derive(Debug, Clone, Copy)]
 pub enum Resize {
     None,
@@ -120,3 +193,11 @@ impl Resize {
         }
     }
 }
+
+fn default_timestamp() -> String {
+    "%R".to_string()
+}
+
+fn default_bool_true() -> bool {
+    true
+}
