@@ -2,8 +2,9 @@ use std::str::FromStr;
 
 use irc::proto;
 use itertools::Itertools;
+use regex::Regex;
 
-use crate::Buffer;
+use crate::{ctcp, message::formatting, Buffer};
 
 #[derive(Debug, Clone, Copy)]
 pub enum Kind {
@@ -18,6 +19,8 @@ pub enum Kind {
     Topic,
     Kick,
     Mode,
+    Format,
+    Away,
     Raw,
 }
 
@@ -26,17 +29,19 @@ impl FromStr for Kind {
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         match s.to_lowercase().as_str() {
-            "join" => Ok(Kind::Join),
+            "join" | "j" => Ok(Kind::Join),
             "motd" => Ok(Kind::Motd),
             "nick" => Ok(Kind::Nick),
             "quit" => Ok(Kind::Quit),
             "msg" => Ok(Kind::Msg),
-            "me" => Ok(Kind::Me),
+            "me" | "describe" => Ok(Kind::Me),
             "whois" => Ok(Kind::Whois),
-            "part" => Ok(Kind::Part),
-            "topic" => Ok(Kind::Topic),
+            "part" | "leave" => Ok(Kind::Part),
+            "topic" | "t" => Ok(Kind::Topic),
             "kick" => Ok(Kind::Kick),
-            "mode" => Ok(Kind::Mode),
+            "mode" | "m" => Ok(Kind::Mode),
+            "format" | "f" => Ok(Kind::Format),
+            "away" => Ok(Kind::Away),
             "raw" => Ok(Kind::Raw),
             _ => Err(()),
         }
@@ -55,8 +60,9 @@ pub enum Command {
     Part(String, Option<String>),
     Topic(String, Option<String>),
     Kick(String, String, Option<String>),
-    Mode(String, Option<String>, Vec<String>),
-    Raw(String, Vec<String>),
+    Mode(String, Option<String>, Option<Vec<String>>),
+    Away(Option<String>),
+    Raw(String),
     Unknown(String, Vec<String>),
 }
 
@@ -70,7 +76,13 @@ pub fn parse(s: &str, buffer: Option<&Buffer>) -> Result<Command, Error> {
     let mut split = rest.split_ascii_whitespace();
 
     let cmd = split.next().ok_or(Error::MissingCommand)?;
+
     let args = split.collect::<Vec<_>>();
+    let raw = if rest.len() == cmd.len() {
+        ""
+    } else {
+        &rest[cmd.len() + 1..]
+    };
 
     let unknown = || {
         Command::Unknown(
@@ -111,22 +123,35 @@ pub fn parse(s: &str, buffer: Option<&Buffer>) -> Result<Command, Error> {
                 Command::Kick(channel, user, comment)
             }),
             Kind::Mode => {
-                let (channel, rest) = args.split_first().ok_or(Error::MissingCommand)?;
-                let (mode, users) = rest.split_first().ok_or(Error::MissingCommand)?;
-
-                Ok(Command::Mode(
-                    channel.to_string(),
-                    Some(mode.to_string()),
-                    users.iter().map(|s| s.to_string()).collect(),
-                ))
+                if let Some((target, rest)) = args.split_first() {
+                    if let Some((mode_string, mode_arguments)) = rest.split_first() {
+                        let mode_string_regex = Regex::new(r"^((\+|\-)[A-Za-z]*)+$").unwrap();
+                        if !mode_string_regex.is_match(mode_string) {
+                            Err(Error::InvalidModeString)
+                        } else {
+                            let mode_arguments: Vec<String> =
+                                mode_arguments.iter().map(|v| v.to_string()).collect();
+                            Ok(Command::Mode(
+                                target.to_string(),
+                                Some(mode_string.to_string()),
+                                (!mode_arguments.is_empty()).then_some(mode_arguments),
+                            ))
+                        }
+                    } else {
+                        Ok(Command::Mode(target.to_string(), None, None))
+                    }
+                } else {
+                    Err(Error::MissingArgs)
+                }
             }
-            Kind::Raw => {
-                let (cmd, args) = args.split_first().ok_or(Error::MissingCommand)?;
-
-                Ok(Command::Raw(
-                    cmd.to_string(),
-                    args.iter().map(|s| s.to_string()).collect(),
-                ))
+            Kind::Away => validated::<0, 1, true>(args, |_, [comment]| Command::Away(comment)),
+            Kind::Raw => Ok(Command::Raw(raw.to_string())),
+            Kind::Format => {
+                if let Some(target) = buffer.and_then(|b| b.target()) {
+                    Ok(Command::Msg(target, formatting::encode(raw, false)))
+                } else {
+                    Ok(unknown())
+                }
             }
         },
         Err(_) => Ok(unknown()),
@@ -183,14 +208,17 @@ impl TryFrom<Command> for proto::Command {
             Command::Quit(comment) => proto::Command::QUIT(comment),
             Command::Msg(target, msg) => proto::Command::PRIVMSG(target, msg),
             Command::Me(target, text) => {
-                proto::Command::PRIVMSG(target, format!("\u{1}ACTION {text}\u{1}"))
+                ctcp::query_command(&ctcp::Command::Action, target, Some(text))
             }
             Command::Whois(channel, user) => proto::Command::WHOIS(channel, user),
             Command::Part(chanlist, reason) => proto::Command::PART(chanlist, reason),
             Command::Topic(channel, topic) => proto::Command::TOPIC(channel, topic),
             Command::Kick(channel, user, comment) => proto::Command::KICK(channel, user, comment),
-            Command::Mode(channel, mode, users) => proto::Command::MODE(channel, mode, users),
-            Command::Raw(command, args) => proto::Command::Unknown(command, args),
+            Command::Mode(target, modestring, modearguments) => {
+                proto::Command::MODE(target, modestring, modearguments)
+            }
+            Command::Away(comment) => proto::Command::AWAY(comment),
+            Command::Raw(raw) => proto::Command::Raw(raw),
             Command::Unknown(command, args) => proto::Command::new(&command, args),
         })
     }
@@ -208,6 +236,10 @@ pub enum Error {
     MissingSlash,
     #[error("missing command")]
     MissingCommand,
+    #[error("missing args")]
+    MissingArgs,
+    #[error("invalid modestring")]
+    InvalidModeString,
 }
 
 fn fmt_incorrect_arg_count(min: usize, max: usize, actual: usize) -> String {
