@@ -1,40 +1,54 @@
-use std::fs::{self, File};
-use std::io::BufReader;
 use std::path::PathBuf;
+use std::string;
+
+use tokio_stream::wrappers::ReadDirStream;
+use tokio_stream::StreamExt;
+
+use rand::prelude::*;
+use rand_chacha::ChaCha8Rng;
 
 use serde::Deserialize;
 use thiserror::Error;
 
 pub use self::buffer::Buffer;
 pub use self::channel::Channel;
-pub use self::dashboard::Dashboard;
-pub use self::keys::Keys;
-pub use self::notification::{Notification, Notifications};
+pub use self::file_transfer::FileTransfer;
+pub use self::keys::Keyboard;
+pub use self::notification::Notifications;
+pub use self::proxy::Proxy;
 pub use self::server::Server;
+pub use self::sidebar::Sidebar;
+use crate::audio::{self, Sound};
+use crate::environment::config_dir;
 use crate::server::Map as ServerMap;
-use crate::theme::Palette;
+use crate::theme::Colors;
 use crate::{environment, Theme};
 
 pub mod buffer;
 pub mod channel;
-pub mod dashboard;
+pub mod file_transfer;
 mod keys;
 pub mod notification;
+pub mod proxy;
 pub mod server;
+pub mod sidebar;
 
-const CONFIG_TEMPLATE: &[u8] = include_bytes!("../../config.yaml");
-const DEFAULT_THEME_FILE_NAME: &str = "ferra.yaml";
+const CONFIG_TEMPLATE: &str = include_str!("../../config.toml");
+const DEFAULT_THEME_FILE_NAME: &str = "ferra.toml";
 
 #[derive(Debug, Clone, Default)]
 pub struct Config {
     pub themes: Themes,
     pub servers: ServerMap,
+    pub proxy: Option<Proxy>,
     pub font: Font,
     pub scale_factor: ScaleFactor,
     pub buffer: Buffer,
-    pub dashboard: Dashboard,
-    pub keys: Keys,
-    pub notifications: Notifications,
+    pub sidebar: Sidebar,
+    pub keyboard: Keyboard,
+    pub notifications: Notifications<Sound>,
+    pub file_transfer: FileTransfer,
+    pub tooltips: bool,
 }
 
 #[derive(Debug, Clone, Copy, Deserialize)]
@@ -91,7 +105,18 @@ impl Config {
         dir
     }
 
-    fn themes_dir() -> PathBuf {
+    pub fn sounds_dir() -> PathBuf {
+        let dir = Self::config_dir().join("sounds");
+
+        if !dir.exists() {
+            std::fs::create_dir_all(dir.as_path())
+                .expect("expected permissions to create sounds folder");
+        }
+
+        dir
+    }
+
+    pub fn themes_dir() -> PathBuf {
         let dir = Self::config_dir().join("themes");
 
         if !dir.exists() {
@@ -106,78 +131,104 @@ impl Config {
         Self::config_dir().join(environment::CONFIG_FILE_NAME)
     }
 
-    pub fn load() -> Result<Self, Error> {
+    pub async fn load() -> Result<Self, Error> {
+        use tokio::fs;
+
         #[derive(Deserialize)]
         pub struct Configuration {
             #[serde(default)]
             pub theme: String,
             pub servers: ServerMap,
+            pub proxy: Option<Proxy>,
             #[serde(default)]
             pub font: Font,
             #[serde(default)]
             pub scale_factor: ScaleFactor,
-            #[serde(default, alias = "new_buffer")]
+            #[serde(default)]
             pub buffer: Buffer,
             #[serde(default)]
-            pub dashboard: Dashboard,
+            pub sidebar: Sidebar,
             #[serde(default)]
-            pub keys: Keys,
+            pub keyboard: Keyboard,
             #[serde(default)]
             pub notifications: Notifications,
+            #[serde(default)]
+            pub file_transfer: FileTransfer,
+            #[serde(default = "default_tooltip")]
+            pub tooltips: bool,
         }
 
         let path = Self::path();
-        let file = File::open(path).map_err(|e| Error::Read(e.to_string()))?;
+        let content = fs::read_to_string(path)
+            .await
+            .map_err(|e| Error::Read(e.to_string()))?;
 
         let Configuration {
             theme,
-            servers,
+            mut servers,
             font,
+            proxy,
             scale_factor,
             buffer,
-            dashboard,
-            keys,
+            sidebar,
+            keyboard,
             notifications,
-        } = serde_yaml::from_reader(BufReader::new(file))
-            .map_err(|e| Error::Parse(e.to_string()))?;
+            file_transfer,
+            tooltips,
+        } = toml::from_str(content.as_ref()).map_err(|e| Error::Parse(e.to_string()))?;
 
-        let themes = Self::load_themes(&theme).unwrap_or_default();
+        servers.read_passwords().await?;
+
+        let loaded_notifications = notifications.load_sounds()?;
+
+        let themes = Self::load_themes(&theme).await.unwrap_or_default();
 
         Ok(Config {
             themes,
             servers,
             font,
+            proxy,
             scale_factor,
             buffer,
-            dashboard,
-            keys,
-            notifications,
+            sidebar,
+            keyboard,
+            notifications: loaded_notifications,
+            file_transfer,
+            tooltips,
         })
     }
 
-    fn load_themes(default_key: &str) -> Result<Themes, Error> {
+    async fn load_themes(default_key: &str) -> Result<Themes, Error> {
+        use tokio::fs;
+
         #[derive(Deserialize)]
-        pub struct Data {
-            #[serde(default)]
-            pub name: String,
-            #[serde(default)]
-            pub palette: Palette,
+        #[serde(untagged)]
+        pub enum Data {
+            V1 {
+                #[serde(rename = "name")]
+                _name: String,
+            },
+            V2(Colors),
         }
 
-        let read_entry = |entry: fs::DirEntry| {
-            let content = fs::read(entry.path())?;
+        let read_entry = |entry: fs::DirEntry| async move {
+            let content = fs::read_to_string(entry.path()).await.ok()?;
 
-            let Data { name, palette } =
-                serde_yaml::from_slice(&content).map_err(|e| Error::Parse(e.to_string()))?;
+            let data: Data = toml::from_str(content.as_ref()).ok()?;
+            let name = entry.path().file_stem()?.to_string_lossy().to_string();
 
-            Ok::<Theme, Error>(Theme::new(name, &palette))
+            match data {
+                Data::V1 { .. } => None,
+                Data::V2(colors) => Some(Theme::new(name, colors)),
+            }
         };
 
         let mut all = vec![];
         let mut default = Theme::default();
         let mut has_halloy_theme = false;
 
-        for entry in fs::read_dir(Self::themes_dir())? {
+        let mut stream = ReadDirStream::new(fs::read_dir(Self::themes_dir()).await?);
+        while let Some(entry) = stream.next().await {
             let Ok(entry) = entry else {
                 continue;
             };
@@ -186,9 +237,11 @@ impl Config {
                 continue;
             };
 
-            if file_name.ends_with(".yaml") {
-                if let Ok(theme) = read_entry(entry) {
-                    if file_name.strip_suffix(".yaml").unwrap_or_default() == default_key {
+            if file_name.ends_with(".toml") {
+                if let Some(theme) = read_entry(entry).await {
+                    if file_name.strip_suffix(".toml").unwrap_or_default() == default_key
+                        || file_name == default_key
+                    {
                         default = theme.clone();
                     }
                     if file_name == DEFAULT_THEME_FILE_NAME {
@@ -207,27 +260,46 @@ impl Config {
         Ok(Themes { default, all })
     }
 
-    pub fn create_template_config() {
+    pub fn create_initial_config() {
         // Checks if a config file is there
         let config_file = Self::path();
         if config_file.exists() {
             return;
         }
 
-        // Create template configuration file.
-        let config_template_file = Self::config_dir().join("config.template.yaml");
-        let _ = fs::write(config_template_file, CONFIG_TEMPLATE);
+        // Generate a unique nick
+        let rand_nick = random_nickname();
+
+        // Replace placeholder nick with unique nick
+        let config_string = CONFIG_TEMPLATE.replace("__NICKNAME__", rand_nick.as_str());
+        let config_bytes = config_string.as_bytes();
+
+        // Create configuration path.
+        let config_path = Self::config_dir().join("config.toml");
+
+        let _ = std::fs::write(config_path, config_bytes);
     }
 }
 
-pub fn create_themes_dir() {
-    const CONTENT: &[u8] = include_bytes!("../../assets/themes/ferra.yaml");
+pub fn random_nickname() -> String {
+    let mut rng = ChaCha8Rng::from_entropy();
+    random_nickname_with_seed(&mut rng)
+}
 
-    // Create default theme file.
-    let file = Config::themes_dir().join(DEFAULT_THEME_FILE_NAME);
-    if !file.exists() {
-        let _ = fs::write(file, CONTENT);
-    }
+pub fn random_nickname_with_seed<R: Rng>(rng: &mut R) -> String {
+    let rand_digit: u16 = rng.gen_range(1000..=9999);
+    let rand_nick = format!("halloy{rand_digit}");
+
+    rand_nick
+}
+
+/// Has YAML configuration file.
+pub fn has_yaml_config() -> bool {
+    config_dir().join("config.yaml").exists()
+}
+
+fn default_tooltip() -> bool {
+    true
 }
 
 #[derive(Debug, Error, Clone)]
@@ -238,6 +310,10 @@ pub enum Error {
     Io(String),
     #[error("{0}")]
     Parse(String),
+    #[error("UTF8 parsing error: {0}")]
+    UI(#[from] string::FromUtf8Error),
+    #[error(transparent)]
+    LoadSounds(#[from] audio::LoadError),
 }
 
 impl From<std::io::Error> for Error {
