@@ -3,7 +3,7 @@ use serde::Deserialize;
 use crate::shortcut::{shortcut, KeyBind, Shortcut};
 
 #[derive(Debug, Clone, Deserialize)]
-pub struct Keys {
+pub struct Keyboard {
     #[serde(default = "KeyBind::move_up")]
     pub move_up: KeyBind,
     #[serde(default = "KeyBind::move_down")]
@@ -22,13 +22,19 @@ pub struct Keys {
     pub cycle_next_buffer: KeyBind,
     #[serde(default = "KeyBind::cycle_previous_buffer")]
     pub cycle_previous_buffer: KeyBind,
+    #[serde(default = "KeyBind::leave_buffer")]
+    pub leave_buffer: KeyBind,
     #[serde(default = "KeyBind::toggle_nick_list")]
     pub toggle_nick_list: KeyBind,
+    #[serde(default = "KeyBind::toggle_sidebar")]
+    pub toggle_sidebar: KeyBind,
     #[serde(default = "KeyBind::command_bar")]
     pub command_bar: KeyBind,
+    #[serde(default = "KeyBind::reload_configuration")]
+    pub reload_configuration: KeyBind,
 }
 
-impl Default for Keys {
+impl Default for Keyboard {
     fn default() -> Self {
         Self {
             move_up: KeyBind::move_up(),
@@ -40,13 +46,16 @@ impl Default for Keys {
             restore_buffer: KeyBind::restore_buffer(),
             cycle_next_buffer: KeyBind::cycle_next_buffer(),
             cycle_previous_buffer: KeyBind::cycle_previous_buffer(),
+            leave_buffer: KeyBind::leave_buffer(),
             toggle_nick_list: KeyBind::toggle_nick_list(),
+            toggle_sidebar: KeyBind::toggle_sidebar(),
             command_bar: KeyBind::command_bar(),
+            reload_configuration: KeyBind::reload_configuration(),
         }
     }
 }
 
-impl Keys {
+impl Keyboard {
     pub fn shortcuts(&self) -> Vec<Shortcut> {
         use crate::shortcut::Command::*;
 
@@ -60,8 +69,11 @@ impl Keys {
             shortcut(self.restore_buffer.clone(), RestoreBuffer),
             shortcut(self.cycle_next_buffer.clone(), CycleNextBuffer),
             shortcut(self.cycle_previous_buffer.clone(), CyclePreviousBuffer),
-            shortcut(self.toggle_nick_list.clone(), ToggleNickList),
+            shortcut(self.leave_buffer.clone(), LeaveBuffer),
+            shortcut(self.toggle_nick_list.clone(), ToggleNicklist),
+            shortcut(self.toggle_sidebar.clone(), ToggleSidebar),
             shortcut(self.command_bar.clone(), CommandBar),
+            shortcut(self.reload_configuration.clone(), ReloadConfiguration),
         ]
     }
 }
