@@ -9,13 +9,24 @@ use crate::{compression, environment};
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct Dashboard {
     pub pane: Pane,
+    #[serde(default)]
+    pub popout_panes: Vec<Pane>,
 }
 
 #[derive(Debug, Clone, Copy, Default, Deserialize, Serialize)]
-pub enum DefaultAction {
+#[serde(rename_all = "kebab-case")]
+pub enum BufferAction {
     #[default]
     NewPane,
     ReplacePane,
+    NewWindow,
+}
+
+#[derive(Debug, Clone, Copy, Default, Deserialize, Serialize)]
+#[serde(rename_all = "kebab-case")]
+pub enum BufferFocusedAction {
+    #[default]
+    ClosePane,
 }
 
 impl Dashboard {
