@@ -6,7 +6,6 @@ use futures::{future, Future, FutureExt};
 use itertools::Itertools;
 use tokio::time::Instant;
 
-use crate::config::buffer::Exclude;
 use crate::history::{self, History};
 use crate::message::{self, Limit};
 use crate::time::Posix;
@@ -167,9 +166,11 @@ impl Manager {
         }
     }
 
-    pub fn record_input(&mut self, input: Input, user: User) {
-        if let Some(message) = input.message(user) {
-            self.record_message(input.server(), message);
+    pub fn record_input(&mut self, input: Input, user: User, channel_users: &[User]) {
+        if let Some(messages) = input.messages(user, channel_users) {
+            for message in messages {
+                self.record_message(input.server(), message);
+            }
         }
 
         if let Some(text) = input.raw() {
@@ -263,7 +264,13 @@ impl Manager {
             .unwrap_or_default()
     }
 
-    pub fn broadcast(&mut self, server: &Server, broadcast: Broadcast, config: &Config) {
+    pub fn broadcast(
+        &mut self,
+        server: &Server,
+        broadcast: Broadcast,
+        config: &Config,
+        sent_time: DateTime<Utc>,
+    ) {
         let map = self.data.map.entry(server.clone()).or_default();
 
         let channels = map
@@ -288,13 +295,15 @@ impl Manager {
             .cloned();
 
         let messages = match broadcast {
-            Broadcast::Connecting => message::broadcast::connecting(),
-            Broadcast::Connected => message::broadcast::connected(),
-            Broadcast::ConnectionFailed { error } => message::broadcast::connection_failed(error),
-            Broadcast::Disconnected { error } => {
-                message::broadcast::disconnected(channels, queries, error)
+            Broadcast::Connecting => message::broadcast::connecting(sent_time),
+            Broadcast::Connected => message::broadcast::connected(sent_time),
+            Broadcast::ConnectionFailed { error } => {
+                message::broadcast::connection_failed(error, sent_time)
             }
-            Broadcast::Reconnected => message::broadcast::reconnected(channels, queries),
+            Broadcast::Disconnected { error } => {
+                message::broadcast::disconnected(channels, queries, error, sent_time)
+            }
+            Broadcast::Reconnected => message::broadcast::reconnected(channels, queries, sent_time),
             Broadcast::Quit {
                 user,
                 comment,
@@ -302,7 +311,14 @@ impl Manager {
             } => {
                 let user_query = queries.find(|nick| user.nickname() == *nick);
 
-                message::broadcast::quit(user_channels, user_query, &user, &comment, config)
+                message::broadcast::quit(
+                    user_channels,
+                    user_query,
+                    &user,
+                    &comment,
+                    config,
+                    sent_time,
+                )
             }
             Broadcast::Nickname {
                 old_nick,
@@ -318,6 +334,7 @@ impl Manager {
                         &old_nick,
                         &new_nick,
                         ourself,
+                        sent_time,
                     )
                 } else {
                     // Otherwise just the query channel of the user w/ nick change
@@ -328,6 +345,7 @@ impl Manager {
                         &old_nick,
                         &new_nick,
                         ourself,
+                        sent_time,
                     )
                 }
             }
@@ -335,7 +353,39 @@ impl Manager {
                 inviter,
                 channel,
                 user_channels,
-            } => message::broadcast::invite(inviter, channel, user_channels),
+            } => message::broadcast::invite(inviter, channel, user_channels, sent_time),
+            Broadcast::ChangeHost {
+                old_user,
+                new_username,
+                new_hostname,
+                ourself,
+                user_channels,
+            } => {
+                if ourself {
+                    // If ourself, broadcast to all query channels (since we are in all of them)
+                    message::broadcast::change_host(
+                        user_channels,
+                        queries,
+                        &old_user,
+                        &new_username,
+                        &new_hostname,
+                        ourself,
+                        sent_time,
+                    )
+                } else {
+                    // Otherwise just the query channel of the user w/ host change
+                    let user_query = queries.find(|nick| old_user.nickname() == *nick);
+                    message::broadcast::change_host(
+                        user_channels,
+                        user_query,
+                        &old_user,
+                        &new_username,
+                        &new_hostname,
+                        ourself,
+                        sent_time,
+                    )
+                }
+            }
         };
 
         messages.into_iter().for_each(|message| {
@@ -449,35 +499,37 @@ impl Data {
             .iter()
             .filter(|message| match message.target.source() {
                 message::Source::Server(Some(source)) => {
-                    if let Some(source_config) = buffer_config.server_messages.get(source) {
-                        match source_config.exclude {
-                            Exclude::All => false,
-                            Exclude::None => true,
-                            Exclude::Smart(seconds) => {
-                                if let Some(nick) = source.nick() {
-                                    !smart_filter_message(
-                                        message,
-                                        &seconds,
-                                        most_recent_messages.get(nick),
-                                    )
-                                } else if let Some(nickname) =
-                                    message.text.split(' ').collect::<Vec<_>>().get(1)
-                                {
-                                    let nick = Nick::from(*nickname);
-
-                                    !smart_filter_message(
-                                        message,
-                                        &seconds,
-                                        most_recent_messages.get(&nick),
-                                    )
-                                } else {
-                                    true
-                                }
+                    if let Some(server_message) = buffer_config.server_messages.get(source) {
+                        // Check if target is a channel, and if included/excluded.
+                        if let message::Target::Channel { channel, .. } = &message.target {
+                            if !server_message.should_send_message(channel.as_ref()) {
+                                return false;
                             }
                         }
-                    } else {
-                        true
+
+                        if let Some(seconds) = server_message.smart {
+                            let nick = match source.nick() {
+                                Some(nick) => nick.clone(),
+                                None => {
+                                    if let Some(nickname) =
+                                        message.plain().and_then(|s| s.split(' ').nth(1))
+                                    {
+                                        Nick::from(nickname)
+                                    } else {
+                                        return true;
+                                    }
+                                }
+                            };
+
+                            return !smart_filter_message(
+                                message,
+                                &seconds,
+                                most_recent_messages.get(&nick),
+                            );
+                        }
                     }
+
+                    true
                 }
                 crate::message::Source::User(message_user) => {
                     most_recent_messages
@@ -485,11 +537,69 @@ impl Data {
 
                     true
                 }
+                message::Source::Internal(message::source::Internal::Status(status)) => {
+                    if let Some(internal_message) = buffer_config.internal_messages.get(status) {
+                        if !internal_message.enabled {
+                            return false;
+                        }
+
+                        if let Some(seconds) = internal_message.smart {
+                            return !smart_filter_internal_message(message, &seconds);
+                        }
+                    }
+
+                    true
+                }
                 _ => true,
             })
             .collect::<Vec<_>>();
 
         let total = filtered.len();
+        let with_access_levels = buffer_config.nickname.show_access_levels;
+
+        let max_nick_chars = buffer_config.nickname.alignment.is_right().then(|| {
+            filtered
+                .iter()
+                .filter_map(|message| {
+                    if let message::Source::User(user) = message.target.source() {
+                        Some(
+                            buffer_config
+                                .nickname
+                                .brackets
+                                .format(user.display(with_access_levels))
+                                .chars()
+                                .count(),
+                        )
+                    } else {
+                        None
+                    }
+                })
+                .max()
+                .unwrap_or_default()
+        });
+
+        let max_prefix_chars = buffer_config.nickname.alignment.is_right().then(|| {
+            if matches!(kind, history::Kind::Channel(_)) {
+                filtered
+                    .iter()
+                    .filter_map(|message| {
+                        message.target.prefix().map(|prefix| {
+                            buffer_config
+                                .status_message_prefix
+                                .brackets
+                                .format(prefix)
+                                .chars()
+                                .count()
+                                + 1
+                        })
+                    })
+                    .max()
+                    .unwrap_or_default()
+            } else {
+                0
+            }
+        });
+
         let limited = with_limit(limit, filtered.into_iter());
 
         let split_at = limited
@@ -503,6 +613,8 @@ impl Data {
             total,
             old_messages: old.to_vec(),
             new_messages: new.to_vec(),
+            max_nick_chars,
+            max_prefix_chars,
         })
     }
 
@@ -565,6 +677,16 @@ fn smart_filter_message(
     duration_seconds > *seconds
 }
 
+fn smart_filter_internal_message(message: &crate::Message, seconds: &i64) -> bool {
+    let current_time = Utc::now();
+
+    let duration_seconds = current_time
+        .signed_duration_since(message.server_time)
+        .num_seconds();
+
+    duration_seconds > *seconds
+}
+
 #[derive(Debug, Clone)]
 pub enum Broadcast {
     Connecting,
@@ -592,4 +714,11 @@ pub enum Broadcast {
         channel: String,
         user_channels: Vec<String>,
     },
+    ChangeHost {
+        old_user: User,
+        new_username: String,
+        new_hostname: String,
+        ourself: bool,
+        user_channels: Vec<String>,
+    },
 }
