@@ -4,15 +4,25 @@ use chrono::Utc;
 use irc::proto;
 use irc::proto::format;
 
+use crate::buffer::AutoFormat;
+use crate::message::formatting;
 use crate::time::Posix;
 use crate::{command, message, Buffer, Command, Message, Server, User};
 
 const INPUT_HISTORY_LENGTH: usize = 100;
 
-pub fn parse(buffer: Buffer, input: &str) -> Result<Input, Error> {
+pub fn parse(buffer: Buffer, auto_format: AutoFormat, input: &str) -> Result<Input, Error> {
     let content = match command::parse(input, Some(&buffer)) {
         Ok(command) => Content::Command(command),
-        Err(command::Error::MissingSlash) => Content::Text(input.to_string()),
+        Err(command::Error::MissingSlash) => {
+            let text = match auto_format {
+                AutoFormat::Disabled => input.to_string(),
+                AutoFormat::Markdown => formatting::encode(input, true),
+                AutoFormat::All => formatting::encode(input, false),
+            };
+
+            Content::Text(text)
+        }
         Err(error) => return Err(Error::Command(error)),
     };
 
@@ -55,12 +65,13 @@ impl Input {
         self.buffer.server()
     }
 
-    pub fn message(&self, user: User) -> Option<Message> {
-        let to_target = |target: String, source| {
-            if proto::is_channel(&target) {
+    pub fn messages(&self, user: User, channel_users: &[User]) -> Option<Vec<Message>> {
+        let to_target = |target: &str, source| {
+            if let Some((prefix, channel)) = proto::parse_channel_from_target(target) {
                 Some(message::Target::Channel {
-                    channel: target,
+                    channel,
                     source,
+                    prefix,
                 })
             } else if let Ok(user) = User::try_from(target) {
                 Some(message::Target::Query {
@@ -75,20 +86,28 @@ impl Input {
         let command = self.content.command(&self.buffer)?;
 
         match command {
-            Command::Msg(target, text) => Some(Message {
+            Command::Msg(targets, text) => Some(
+                targets
+                    .split(',')
+                    .filter_map(|target| to_target(target, message::Source::User(user.clone())))
+                    .map(|target| Message {
+                        received_at: Posix::now(),
+                        server_time: Utc::now(),
+                        direction: message::Direction::Sent,
+                        target,
+                        content: message::parse_fragments(text.clone(), channel_users),
+                        id: None,
+                    })
+                    .collect(),
+            ),
+            Command::Me(target, action) => Some(vec![Message {
                 received_at: Posix::now(),
                 server_time: Utc::now(),
                 direction: message::Direction::Sent,
-                target: to_target(target, message::Source::User(user))?,
-                text,
-            }),
-            Command::Me(target, action) => Some(Message {
-                received_at: Posix::now(),
-                server_time: Utc::now(),
-                direction: message::Direction::Sent,
-                target: to_target(target, message::Source::Action)?,
-                text: message::action_text(user.nickname(), &action),
-            }),
+                target: to_target(&target, message::Source::Action)?,
+                content: message::action_text(user.nickname(), Some(&action)),
+                id: None,
+            }]),
             _ => None,
         }
     }
