@@ -11,18 +11,26 @@ pub use self::pane::Pane;
 pub use self::server::Server;
 pub use self::shortcut::Shortcut;
 pub use self::theme::Theme;
+pub use self::url::Url;
 pub use self::user::User;
+pub use self::version::Version;
+pub use self::window::Window;
 
+pub mod audio;
 pub mod buffer;
 pub mod channel;
 pub mod client;
 pub mod command;
 mod compression;
 pub mod config;
+pub mod ctcp;
 pub mod dashboard;
+pub mod dcc;
 pub mod environment;
+pub mod file_transfer;
 pub mod history;
 pub mod input;
+pub mod isupport;
 pub mod log;
 pub mod message;
 pub mod mode;
@@ -32,4 +40,7 @@ pub mod shortcut;
 pub mod stream;
 pub mod theme;
 pub mod time;
+pub mod url;
 pub mod user;
+pub mod version;
+pub mod window;
