@@ -1,7 +1,8 @@
 //! Generate messages that can be broadcast into every buffer
-use chrono::Utc;
+use chrono::{DateTime, Utc};
 
-use super::{source, Direction, Message, Source, Target};
+use super::{parse_fragments, plain, source, Content, Direction, Message, Source, Target};
+use crate::config::buffer::UsernameFormat;
 use crate::time::Posix;
 use crate::user::Nick;
 use crate::{Config, User};
@@ -16,15 +17,17 @@ fn expand(
     queries: impl IntoIterator<Item = Nick>,
     include_server: bool,
     cause: Cause,
-    text: String,
+    content: Content,
+    sent_time: DateTime<Utc>,
 ) -> Vec<Message> {
-    let message = |target, text| -> Message {
+    let message = |target, content| -> Message {
         Message {
             received_at: Posix::now(),
-            server_time: Utc::now(),
+            server_time: sent_time,
             direction: Direction::Received,
             target,
-            text,
+            content,
+            id: None,
         }
     };
 
@@ -40,8 +43,9 @@ fn expand(
                 Target::Channel {
                     channel,
                     source: source.clone(),
+                    prefix: None,
                 },
-                text.clone(),
+                content.clone(),
             )
         })
         .chain(queries.into_iter().map(|nick| {
@@ -50,7 +54,7 @@ fn expand(
                     nick,
                     source: source.clone(),
                 },
-                text.clone(),
+                content.clone(),
             )
         }))
         .chain(include_server.then(|| {
@@ -58,54 +62,79 @@ fn expand(
                 Target::Server {
                     source: source.clone(),
                 },
-                text.clone(),
+                content.clone(),
             )
         }))
         .collect()
 }
 
-pub fn connecting() -> Vec<Message> {
-    let text = " ∙ connecting to server...".into();
-    expand([], [], true, Cause::Status(source::Status::Success), text)
+pub fn connecting(sent_time: DateTime<Utc>) -> Vec<Message> {
+    let content = plain("connecting to server...".into());
+    expand(
+        [],
+        [],
+        true,
+        Cause::Status(source::Status::Success),
+        content,
+        sent_time,
+    )
 }
 
-pub fn connected() -> Vec<Message> {
-    let text = " ∙ connected".into();
-    expand([], [], true, Cause::Status(source::Status::Success), text)
+pub fn connected(sent_time: DateTime<Utc>) -> Vec<Message> {
+    let content = plain("connected".into());
+    expand(
+        [],
+        [],
+        true,
+        Cause::Status(source::Status::Success),
+        content,
+        sent_time,
+    )
 }
 
-pub fn connection_failed(error: String) -> Vec<Message> {
-    let text = format!(" ∙ connection to server failed ({error})");
-    expand([], [], true, Cause::Status(source::Status::Error), text)
+pub fn connection_failed(error: String, sent_time: DateTime<Utc>) -> Vec<Message> {
+    let content = plain(format!("connection to server failed ({error})"));
+    expand(
+        [],
+        [],
+        true,
+        Cause::Status(source::Status::Error),
+        content,
+        sent_time,
+    )
 }
 
 pub fn disconnected(
     channels: impl IntoIterator<Item = String>,
     queries: impl IntoIterator<Item = Nick>,
     error: Option<String>,
+    sent_time: DateTime<Utc>,
 ) -> Vec<Message> {
     let error = error.map(|error| format!(" ({error})")).unwrap_or_default();
-    let text = format!(" ∙ connection to server lost{error}");
+    let content = plain(format!("connection to server lost{error}"));
     expand(
         channels,
         queries,
         true,
         Cause::Status(source::Status::Error),
-        text,
+        content,
+        sent_time,
     )
 }
 
 pub fn reconnected(
     channels: impl IntoIterator<Item = String>,
     queries: impl IntoIterator<Item = Nick>,
+    sent_time: DateTime<Utc>,
 ) -> Vec<Message> {
-    let text = " ∙ connection to server restored".into();
+    let content = plain("connection to server restored".into());
     expand(
         channels,
         queries,
         true,
         Cause::Status(source::Status::Success),
-        text,
+        content,
+        sent_time,
     )
 }
 
@@ -115,14 +144,19 @@ pub fn quit(
     user: &User,
     comment: &Option<String>,
     config: &Config,
+    sent_time: DateTime<Utc>,
 ) -> Vec<Message> {
     let comment = comment
         .as_ref()
         .map(|comment| format!(" ({comment})"))
         .unwrap_or_default();
-    let text = format!(
-        "⟵ {} has quit{comment}",
-        user.formatted(config.buffer.server_messages.quit.username_format)
+
+    let content = parse_fragments(
+        format!(
+            "⟵ {} has quit{comment}",
+            user.formatted(config.buffer.server_messages.quit.username_format)
+        ),
+        &[],
     );
 
     expand(
@@ -133,7 +167,8 @@ pub fn quit(
             source::server::Kind::Quit,
             Some(user.nickname().to_owned()),
         ))),
-        text,
+        content,
+        sent_time,
     )
 }
 
@@ -143,22 +178,64 @@ pub fn nickname(
     old_nick: &Nick,
     new_nick: &Nick,
     ourself: bool,
+    sent_time: DateTime<Utc>,
 ) -> Vec<Message> {
-    let text = if ourself {
-        format!(" ∙ You're now known as {new_nick}")
+    let content = if ourself {
+        plain(format!("You're now known as {new_nick}"))
     } else {
-        format!(" ∙ {old_nick} is now known as {new_nick}")
+        plain(format!("{old_nick} is now known as {new_nick}"))
     };
 
-    expand(channels, queries, false, Cause::Server(None), text)
+    expand(
+        channels,
+        queries,
+        false,
+        Cause::Server(None),
+        content,
+        sent_time,
+    )
 }
 
 pub fn invite(
     inviter: Nick,
     channel: String,
     channels: impl IntoIterator<Item = String>,
+    sent_time: DateTime<Utc>,
 ) -> Vec<Message> {
-    let text = format!(" ∙ {inviter} invited you to join {channel}");
+    let content = plain(format!("{inviter} invited you to join {channel}"));
 
-    expand(channels, [], false, Cause::Server(None), text)
+    expand(channels, [], false, Cause::Server(None), content, sent_time)
+}
+
+pub fn change_host(
+    channels: impl IntoIterator<Item = String>,
+    queries: impl IntoIterator<Item = Nick>,
+    old_user: &User,
+    new_username: &str,
+    new_hostname: &str,
+    ourself: bool,
+    sent_time: DateTime<Utc>,
+) -> Vec<Message> {
+    let content = if ourself {
+        plain(format!(
+            "You've changed host to {new_username}@{new_hostname}",
+        ))
+    } else {
+        plain(format!(
+            "{} changed host to {new_username}@{new_hostname}",
+            old_user.formatted(UsernameFormat::Full)
+        ))
+    };
+
+    expand(
+        channels,
+        queries,
+        false,
+        Cause::Server(Some(source::Server::new(
+            source::server::Kind::ChangeHost,
+            Some(old_user.nickname().to_owned()),
+        ))),
+        content,
+        sent_time,
+    )
 }
