@@ -24,7 +24,8 @@ pub enum Command {
     ERROR(String),
 
     /* Channel Operations */
-    /// <channel>{,<channel>} [<key>{,<key>}]
+    /// <channel>{,<channel>} [<key>{,<key>}] (send)
+    /// <channel>{,<channel>} [<accountname>] (receive [extended-join])
     JOIN(String, Option<String>),
     /// <channel>{,<channel>} [<reason>]
     PART(String, Option<String>),
@@ -60,7 +61,7 @@ pub enum Command {
     /// None
     INFO,
     /// <target> [<modestring> [<mode arguments>...]]
-    MODE(String, Option<String>, Vec<String>),
+    MODE(String, Option<String>, Option<Vec<String>>),
 
     /* Sending Messages */
     /// <target>{,<target>} <text to be sent>
@@ -69,8 +70,8 @@ pub enum Command {
     NOTICE(String, String),
 
     /* User-Based Queries */
-    /// <mask>
-    WHO(String),
+    /// <mask> [%<fields>[,<token>]]
+    WHO(String, Option<String>, Option<String>),
     /// [<target>] <nick>
     WHOIS(Option<String>, String),
     /// <nick> [<count>]
@@ -94,11 +95,28 @@ pub enum Command {
     /// <text>
     WALLOPS(String),
 
+    /// <accountname>
+    ACCOUNT(String),
     /* IRC extensions */
     BATCH(String, Vec<String>),
+    /// <new_username> <new_hostname>
+    CHGHOST(String, String),
+    /// <nickname> <channel> :<message>
+    CNOTICE(String, String, String),
+    /// <nickname> <channel> :<message>
+    CPRIVMSG(String, String, String),
+    /// <channel> [<message>]
+    KNOCK(String, Option<String>),
+    /// <subcommand> [<targets>]
+    MONITOR(String, Option<String>),
+    /// <msgtarget>
+    TAGMSG(String),
+    /// <nickname>
+    USERIP(String),
 
     Numeric(Numeric, Vec<String>),
     Unknown(String, Vec<String>),
+    Raw(String),
 }
 
 impl Command {
@@ -161,10 +179,10 @@ impl Command {
             "STATS" if len > 0 => STATS(req!(), opt!()),
             "HELP" => HELP(opt!()),
             "INFO" => INFO,
-            "MODE" if len > 0 => MODE(req!(), opt!(), params.collect()),
+            "MODE" if len > 0 => MODE(req!(), opt!(), Some(params.collect())),
             "PRIVMSG" if len > 1 => PRIVMSG(req!(), req!()),
             "NOTICE" if len > 1 => NOTICE(req!(), req!()),
-            "WHO" if len > 0 => WHO(req!()),
+            "WHO" if len > 0 => WHO(req!(), opt!(), opt!()),
             "WHOIS" => {
                 let a = req!();
                 match opt!() {
@@ -181,7 +199,15 @@ impl Command {
             "LINKS" => LINKS,
             "USERHOST" => USERHOST(params.collect()),
             "WALLOPS" if len > 0 => WALLOPS(req!()),
+            "ACCOUNT" if len > 0 => ACCOUNT(req!()),
             "BATCH" if len > 0 => BATCH(req!(), params.collect()),
+            "CHGHOST" if len > 1 => CHGHOST(req!(), req!()),
+            "CNOTICE" if len > 2 => CNOTICE(req!(), req!(), req!()),
+            "CPRIVMSG" if len > 2 => CPRIVMSG(req!(), req!(), req!()),
+            "KNOCK" if len > 0 => KNOCK(req!(), opt!()),
+            "MONITOR" if len > 0 => MONITOR(req!(), opt!()),
+            "TAGMSG" if len > 0 => TAGMSG(req!()),
+            "USERIP" if len > 0 => USERIP(req!()),
             _ => Self::Unknown(tag, params.collect()),
         }
     }
@@ -214,10 +240,15 @@ impl Command {
             Command::STATS(a, b) => std::iter::once(a).chain(b).collect(),
             Command::HELP(a) => a.into_iter().collect(),
             Command::INFO => vec![],
-            Command::MODE(a, b, c) => std::iter::once(a).chain(b).chain(c).collect(),
+            Command::MODE(a, b, c) => std::iter::once(a)
+                .chain(b)
+                .chain(c.into_iter().flatten())
+                .collect(),
             Command::PRIVMSG(a, b) => vec![a, b],
             Command::NOTICE(a, b) => vec![a, b],
-            Command::WHO(a) => vec![a],
+            Command::WHO(a, b, c) => std::iter::once(a)
+                .chain(b.map(|b| c.map_or_else(|| format!("%{}", b), |c| format!("%{},{}", b, c))))
+                .collect(),
             Command::WHOIS(a, b) => a.into_iter().chain(Some(b)).collect(),
             Command::WHOWAS(a, b) => std::iter::once(a).chain(b).collect(),
             Command::KILL(a, b) => vec![a, b],
@@ -228,9 +259,18 @@ impl Command {
             Command::LINKS => vec![],
             Command::USERHOST(params) => params,
             Command::WALLOPS(a) => vec![a],
+            Command::ACCOUNT(a) => vec![a],
             Command::BATCH(a, rest) => std::iter::once(a).chain(rest).collect(),
+            Command::CHGHOST(a, b) => vec![a, b],
+            Command::CNOTICE(a, b, c) => vec![a, b, c],
+            Command::CPRIVMSG(a, b, c) => vec![a, b, c],
+            Command::KNOCK(a, b) => std::iter::once(a).chain(b).collect(),
+            Command::MONITOR(a, b) => std::iter::once(a).chain(b).collect(),
+            Command::TAGMSG(a) => vec![a],
+            Command::USERIP(a) => vec![a],
             Command::Numeric(_, params) => params,
             Command::Unknown(_, params) => params,
+            Command::Raw(_) => vec![],
         }
     }
 
@@ -267,7 +307,7 @@ impl Command {
             MODE(_, _, _) => "MODE".to_string(),
             PRIVMSG(_, _) => "PRIVMSG".to_string(),
             NOTICE(_, _) => "NOTICE".to_string(),
-            WHO(_) => "WHO".to_string(),
+            WHO(_, _, _) => "WHO".to_string(),
             WHOIS(_, _) => "WHOIS".to_string(),
             WHOWAS(_, _) => "WHOWAS".to_string(),
             KILL(_, _) => "KILL".to_string(),
@@ -278,9 +318,18 @@ impl Command {
             LINKS => "LINKS".to_string(),
             USERHOST(_) => "USERHOST".to_string(),
             WALLOPS(_) => "WALLOPS".to_string(),
+            ACCOUNT(_) => "ACCOUNT".to_string(),
             BATCH(_, _) => "BATCH".to_string(),
+            CHGHOST(_, _) => "CHGHOST".to_string(),
+            CNOTICE(_, _, _) => "CNOTICE".to_string(),
+            CPRIVMSG(_, _, _) => "CPRIVMSG".to_string(),
+            KNOCK(_, _) => "KNOCK".to_string(),
+            MONITOR(_, _) => "MONITOR".to_string(),
+            TAGMSG(_) => "TAGMSG".to_string(),
+            USERIP(_) => "USERIP".to_string(),
             Numeric(numeric, _) => format!("{:03}", *numeric as u16),
             Unknown(tag, _) => tag.clone(),
+            Raw(_) => "".to_string(),
         }
     }
 }
@@ -347,6 +396,7 @@ pub enum Numeric {
     RPL_ENDOFEXCEPTLIST = 349,
     RPL_VERSION = 351,
     RPL_NAMREPLY = 353,
+    RPL_WHOSPCRPL = 354,
     RPL_ENDOFNAMES = 366,
     RPL_LINKS = 364,
     RPL_ENDOFLINKS = 365,
@@ -394,6 +444,7 @@ pub enum Numeric {
     ERR_BANNEDFROMCHAN = 474,
     ERR_BADCHANNELKEY = 475,
     ERR_BADCHANMASK = 476,
+    ERR_NOCHANMODES = 477,
     ERR_NOPRIVILEGES = 481,
     ERR_CHANOPRIVSNEEDED = 482,
     ERR_CANTKILLSERVER = 483,
@@ -410,6 +461,11 @@ pub enum Numeric {
     RPL_HELPTXT = 705,
     RPL_ENDOFHELP = 706,
     ERR_NOPRIVS = 723,
+    RPL_MONONLINE = 730,
+    RPL_MONOFFLINE = 731,
+    RPL_MONLIST = 732,
+    RPL_ENDOFMONLIST = 733,
+    ERR_MONLISTFULL = 734,
     RPL_LOGGEDIN = 900,
     RPL_LOGGEDOUT = 901,
     ERR_NICKLOCKED = 902,
@@ -486,6 +542,7 @@ impl TryFrom<u16> for Numeric {
             349 => RPL_ENDOFEXCEPTLIST,
             351 => RPL_VERSION,
             353 => RPL_NAMREPLY,
+            354 => RPL_WHOSPCRPL,
             366 => RPL_ENDOFNAMES,
             364 => RPL_LINKS,
             365 => RPL_ENDOFLINKS,
@@ -533,6 +590,7 @@ impl TryFrom<u16> for Numeric {
             474 => ERR_BANNEDFROMCHAN,
             475 => ERR_BADCHANNELKEY,
             476 => ERR_BADCHANMASK,
+            477 => ERR_NOCHANMODES,
             481 => ERR_NOPRIVILEGES,
             482 => ERR_CHANOPRIVSNEEDED,
             483 => ERR_CANTKILLSERVER,
@@ -549,6 +607,11 @@ impl TryFrom<u16> for Numeric {
             705 => RPL_HELPTXT,
             706 => RPL_ENDOFHELP,
             723 => ERR_NOPRIVS,
+            730 => RPL_MONONLINE,
+            731 => RPL_MONOFFLINE,
+            732 => RPL_MONLIST,
+            733 => RPL_ENDOFMONLIST,
+            734 => ERR_MONLISTFULL,
             900 => RPL_LOGGEDIN,
             901 => RPL_LOGGEDOUT,
             902 => ERR_NICKLOCKED,
