@@ -1,15 +1,19 @@
 pub use data::buffer::Settings;
 use data::user::Nick;
-use data::{buffer, history, Config};
-use iced::Command;
+use data::{buffer, file_transfer, history, Config};
+use iced::Task;
 
 use self::channel::Channel;
+use self::file_transfers::FileTransfers;
 use self::query::Query;
 use self::server::Server;
+use crate::screen::dashboard::sidebar;
 use crate::widget::Element;
+use crate::Theme;
 
 pub mod channel;
 pub mod empty;
+pub mod file_transfers;
 mod input_view;
 pub mod query;
 mod scroll_view;
@@ -22,6 +26,7 @@ pub enum Buffer {
     Channel(Channel),
     Server(Server),
     Query(Query),
+    FileTransfers(FileTransfers),
 }
 
 #[derive(Debug, Clone)]
@@ -29,11 +34,13 @@ pub enum Message {
     Channel(channel::Message),
     Server(server::Message),
     Query(query::Message),
+    FileTransfers(file_transfers::Message),
 }
 
 #[derive(Debug, Clone)]
 pub enum Event {
     UserContext(user_context::Event),
+    OpenChannel(String),
 }
 
 impl Buffer {
@@ -41,12 +48,13 @@ impl Buffer {
         Self::Empty
     }
 
-    pub fn data(&self) -> Option<data::Buffer> {
+    pub fn data(&self) -> Option<&data::Buffer> {
         match self {
             Buffer::Empty => None,
-            Buffer::Channel(state) => Some(state.buffer()),
-            Buffer::Server(state) => Some(state.buffer()),
-            Buffer::Query(state) => Some(state.buffer()),
+            Buffer::Channel(state) => Some(&state.buffer),
+            Buffer::Server(state) => Some(&state.buffer),
+            Buffer::Query(state) => Some(&state.buffer),
+            Buffer::FileTransfers(_) => None,
         }
     }
 
@@ -55,62 +63,81 @@ impl Buffer {
         message: Message,
         clients: &mut data::client::Map,
         history: &mut history::Manager,
-    ) -> (Command<Message>, Option<Event>) {
+        file_transfers: &mut file_transfer::Manager,
+        config: &Config,
+    ) -> (Task<Message>, Option<Event>) {
         match (self, message) {
             (Buffer::Channel(state), Message::Channel(message)) => {
-                let (command, event) = state.update(message, clients, history);
+                let (command, event) = state.update(message, clients, history, config);
 
                 let event = event.map(|event| match event {
                     channel::Event::UserContext(event) => Event::UserContext(event),
+                    channel::Event::OpenChannel(channel) => Event::OpenChannel(channel),
                 });
 
                 (command.map(Message::Channel), event)
             }
             (Buffer::Server(state), Message::Server(message)) => {
-                let command = state.update(message, clients, history);
+                let (command, event) = state.update(message, clients, history, config);
 
-                (command.map(Message::Server), None)
+                let event = event.map(|event| match event {
+                    server::Event::UserContext(event) => Event::UserContext(event),
+                    server::Event::OpenChannel(channel) => Event::OpenChannel(channel),
+                });
+
+                (command.map(Message::Server), event)
             }
             (Buffer::Query(state), Message::Query(message)) => {
-                let (command, event) = state.update(message, clients, history);
+                let (command, event) = state.update(message, clients, history, config);
 
                 let event = event.map(|event| match event {
                     query::Event::UserContext(event) => Event::UserContext(event),
+                    query::Event::OpenChannel(channel) => Event::OpenChannel(channel),
                 });
 
                 (command.map(Message::Query), event)
             }
-            _ => (Command::none(), None),
+            (Buffer::FileTransfers(state), Message::FileTransfers(message)) => {
+                let command = state.update(message, file_transfers, config);
+
+                (command.map(Message::FileTransfers), None)
+            }
+            _ => (Task::none(), None),
         }
     }
 
     pub fn view<'a>(
         &'a self,
         clients: &'a data::client::Map,
+        file_transfers: &'a file_transfer::Manager,
         history: &'a history::Manager,
         settings: &'a buffer::Settings,
         config: &'a Config,
+        theme: &'a Theme,
         is_focused: bool,
+        sidebar: &'a sidebar::Sidebar,
     ) -> Element<'a, Message> {
         match self {
-            Buffer::Empty => empty::view(),
-            Buffer::Channel(state) => {
-                channel::view(
-                    state,
-                    clients,
-                    history,
-                    &settings.channel,
-                    config,
-                    is_focused,
-                )
-                .map(Message::Channel)
-            }
+            Buffer::Empty => empty::view(config, sidebar),
+            Buffer::Channel(state) => channel::view(
+                state,
+                clients,
+                history,
+                &settings.channel,
+                config,
+                theme,
+                is_focused,
+            )
+            .map(Message::Channel),
             Buffer::Server(state) => {
-                server::view(state, clients, history, config, is_focused)
+                server::view(state, clients, history, config, theme, is_focused)
                     .map(Message::Server)
             }
             Buffer::Query(state) => {
-                query::view(state, clients, history, config, is_focused).map(Message::Query)
+                query::view(state, clients, history, config, theme, is_focused).map(Message::Query)
+            }
+            Buffer::FileTransfers(state) => {
+                file_transfers::view(state, file_transfers).map(Message::FileTransfers)
             }
         }
     }
@@ -135,21 +162,21 @@ impl Buffer {
         }
     }
 
-    pub fn focus(&self) -> Command<Message> {
+    pub fn focus(&self) -> Task<Message> {
         match self {
-            Buffer::Empty => Command::none(),
+            Buffer::Empty | Buffer::FileTransfers(_) => Task::none(),
             Buffer::Channel(channel) => channel.focus().map(Message::Channel),
             Buffer::Server(server) => server.focus().map(Message::Server),
             Buffer::Query(query) => query.focus().map(Message::Query),
         }
     }
 
-    pub fn reset(&self) -> Command<Message> {
+    pub fn reset(&mut self) {
         match self {
-            Buffer::Empty => Command::none(),
-            Buffer::Channel(channel) => channel.reset().map(Message::Channel),
-            Buffer::Server(server) => server.reset().map(Message::Server),
-            Buffer::Query(query) => query.reset().map(Message::Query),
+            Buffer::Empty | Buffer::FileTransfers(_) => {}
+            Buffer::Channel(channel) => channel.reset(),
+            Buffer::Server(server) => server.reset(),
+            Buffer::Query(query) => query.reset(),
         }
     }
 
@@ -157,10 +184,10 @@ impl Buffer {
         &mut self,
         nick: Nick,
         history: &mut history::Manager,
-    ) -> Command<Message> {
-        if let Some(buffer) = self.data() {
+    ) -> Task<Message> {
+        if let Some(buffer) = self.data().cloned() {
             match self {
-                Buffer::Empty | Buffer::Server(_) => Command::none(),
+                Buffer::Empty | Buffer::Server(_) | Buffer::FileTransfers(_) => Task::none(),
                 Buffer::Channel(channel) => channel
                     .input_view
                     .insert_user(nick, buffer, history)
@@ -171,13 +198,13 @@ impl Buffer {
                     .map(|message| Message::Query(query::Message::InputView(message))),
             }
         } else {
-            Command::none()
+            Task::none()
         }
     }
 
-    pub fn scroll_to_start(&mut self) -> Command<Message> {
+    pub fn scroll_to_start(&mut self) -> Task<Message> {
         match self {
-            Buffer::Empty => Command::none(),
+            Buffer::Empty | Buffer::FileTransfers(_) => Task::none(),
             Buffer::Channel(channel) => channel
                 .scroll_view
                 .scroll_to_start()
@@ -193,9 +220,9 @@ impl Buffer {
         }
     }
 
-    pub fn scroll_to_end(&mut self) -> Command<Message> {
+    pub fn scroll_to_end(&mut self) -> Task<Message> {
         match self {
-            Buffer::Empty => Command::none(),
+            Buffer::Empty | Buffer::FileTransfers(_) => Task::none(),
             Buffer::Channel(channel) => channel
                 .scroll_view
                 .scroll_to_end()
