@@ -1,13 +1,13 @@
-use data::message::Limit;
+use data::message::{self, Limit};
 use data::server::Server;
 use data::user::Nick;
 use data::{history, time, Config};
-use iced::widget::{column, container, horizontal_rule, row, text, scrollable, Scrollable};
-use iced::{Command, Length};
+use iced::widget::{column, container, horizontal_rule, row, scrollable, text, Scrollable};
+use iced::{padding, Length, Task};
 
 use super::user_context;
-use crate::theme;
-use crate::widget::Element;
+use crate::widget::{Element, MESSAGE_MARKER_TEXT};
+use crate::{font, theme};
 
 #[derive(Debug, Clone)]
 pub enum Message {
@@ -19,11 +19,13 @@ pub enum Message {
         viewport: scrollable::Viewport,
     },
     UserContext(user_context::Message),
+    Link(message::Link),
 }
 
 #[derive(Debug, Clone)]
 pub enum Event {
     UserContext(user_context::Event),
+    OpenChannel(String),
 }
 
 #[derive(Debug, Clone, Copy)]
@@ -38,12 +40,14 @@ pub fn view<'a>(
     kind: Kind,
     history: &'a history::Manager,
     config: &'a Config,
-    format: impl Fn(&'a data::Message) -> Option<Element<'a, Message>> + 'a,
+    format: impl Fn(&'a data::Message, Option<f32>, Option<f32>) -> Option<Element<'a, Message>> + 'a,
 ) -> Element<'a, Message> {
     let Some(history::View {
         total,
         old_messages,
         new_messages,
+        max_nick_chars,
+        max_prefix_chars,
     }) = (match kind {
         Kind::Server(server) => {
             history.get_server_messages(server, Some(state.limit), &config.buffer)
@@ -69,13 +73,22 @@ pub fn view<'a>(
         .unwrap_or_else(time::Posix::now);
     let status = state.status;
 
+    let max_nick_width = max_nick_chars.map(|len| {
+        font::width_from_chars(
+            usize::max(len, MESSAGE_MARKER_TEXT.chars().count()),
+            &config.font,
+        )
+    });
+
+    let max_prefix_width = max_prefix_chars.map(|len| font::width_from_chars(len, &config.font));
+
     let old = old_messages
         .into_iter()
-        .filter_map(&format)
+        .filter_map(|message| format(message, max_nick_width, max_prefix_width))
         .collect::<Vec<_>>();
     let new = new_messages
         .into_iter()
-        .filter_map(format)
+        .filter_map(|message| format(message, max_nick_width, max_prefix_width))
         .collect::<Vec<_>>();
 
     let show_divider = !new.is_empty() || matches!(status, Status::Idle(Anchor::Bottom));
@@ -86,40 +99,38 @@ pub fn view<'a>(
         let divider = row![
             container(horizontal_rule(1))
                 .width(Length::Fill)
-                .padding([0, 6, 0, 0]),
+                .padding(padding::right(6)),
             text("backlog")
                 .size(font_size)
-                .style(theme::text::transparent),
+                .style(theme::text::secondary),
             container(horizontal_rule(1))
                 .width(Length::Fill)
-                .padding([0, 0, 0, 6])
+                .padding(padding::left(6))
         ]
         .padding(2)
-        .align_items(iced::Alignment::Center);
+        .align_y(iced::Alignment::Center);
 
         column![column(old), divider, column(new)]
     } else {
         column![column(old), column(new)]
     };
 
-    Scrollable::with_direction(
-        container(content).width(Length::Fill).padding([0, 8]),
-        scrollable::Direction::Vertical(
-            scrollable::Properties::default()
-                .alignment(status.alignment())
+    Scrollable::new(container(content).width(Length::Fill).padding([0, 8]))
+        .direction(scrollable::Direction::Vertical(
+            scrollable::Scrollbar::default()
+                .anchor(status.alignment())
                 .width(5)
                 .scroller_width(5),
-        ),
-    )
-    .on_scroll(move |viewport| Message::Scrolled {
-        count,
-        remaining,
-        oldest,
-        status,
-        viewport,
-    })
-    .id(state.scrollable.clone())
-    .into()
+        ))
+        .on_scroll(move |viewport| Message::Scrolled {
+            count,
+            remaining,
+            oldest,
+            status,
+            viewport,
+        })
+        .id(state.scrollable.clone())
+        .into()
 }
 
 #[derive(Debug, Clone)]
@@ -144,7 +155,7 @@ impl State {
         Self::default()
     }
 
-    pub fn update(&mut self, message: Message) -> (Command<Message>, Option<Event>) {
+    pub fn update(&mut self, message: Message) -> (Task<Message>, Option<Event>) {
         match message {
             Message::Scrolled {
                 count,
@@ -213,16 +224,30 @@ impl State {
             }
             Message::UserContext(message) => {
                 return (
-                    Command::none(),
-                    Some(Event::UserContext(user_context::update(message))),
+                    Task::none(),
+                    user_context::update(message).map(Event::UserContext),
                 );
             }
+            Message::Link(message::Link::Channel(channel)) => {
+                return (Task::none(), Some(Event::OpenChannel(channel)))
+            }
+            Message::Link(message::Link::Url(url)) => {
+                let _ = open::that_detached(url);
+            }
+            Message::Link(message::Link::User(user)) => {
+                return (
+                    Task::none(),
+                    Some(Event::UserContext(user_context::Event::SingleClick(
+                        user.nickname().to_owned(),
+                    ))),
+                )
+            }
         }
 
-        (Command::none(), None)
+        (Task::none(), None)
     }
 
-    pub fn scroll_to_start(&mut self) -> Command<Message> {
+    pub fn scroll_to_start(&mut self) -> Task<Message> {
         self.status = Status::Idle(Anchor::Top);
         self.limit = Limit::top();
         scrollable::scroll_to(
@@ -231,7 +256,7 @@ impl State {
         )
     }
 
-    pub fn scroll_to_end(&mut self) -> Command<Message> {
+    pub fn scroll_to_end(&mut self) -> Task<Message> {
         self.status = Status::Idle(Anchor::Bottom);
         self.limit = Limit::bottom();
         scrollable::scroll_to(
@@ -263,16 +288,16 @@ impl Status {
         }
     }
 
-    fn alignment(self) -> scrollable::Alignment {
+    fn alignment(self) -> scrollable::Anchor {
         match self {
             Status::Idle(anchor) => match anchor {
-                Anchor::Top => scrollable::Alignment::Start,
-                Anchor::Bottom => scrollable::Alignment::End,
+                Anchor::Top => scrollable::Anchor::Start,
+                Anchor::Bottom => scrollable::Anchor::End,
             },
-            Status::Unlocked(_) => scrollable::Alignment::Start,
+            Status::Unlocked(_) => scrollable::Anchor::Start,
             Status::Loading(anchor) => match anchor {
-                Anchor::Top => scrollable::Alignment::Start,
-                Anchor::Bottom => scrollable::Alignment::End,
+                Anchor::Top => scrollable::Anchor::Start,
+                Anchor::Bottom => scrollable::Anchor::End,
             },
         }
     }
@@ -293,15 +318,15 @@ impl Status {
 
     fn is_top(self, relative_offset: f32) -> bool {
         match self.alignment() {
-            scrollable::Alignment::Start => relative_offset == 0.0,
-            scrollable::Alignment::End => relative_offset == 1.0,
+            scrollable::Anchor::Start => relative_offset == 0.0,
+            scrollable::Anchor::End => relative_offset == 1.0,
         }
     }
 
     fn is_bottom(self, relative_offset: f32) -> bool {
         match self.alignment() {
-            scrollable::Alignment::Start => relative_offset == 1.0,
-            scrollable::Alignment::End => relative_offset == 0.0,
+            scrollable::Anchor::Start => relative_offset == 1.0,
+            scrollable::Anchor::End => relative_offset == 0.0,
         }
     }
 
