@@ -2,21 +2,24 @@ use iced::{event, keyboard, window, Subscription};
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum Event {
-    CloseRequested,
     Copy,
     Escape,
     Home,
     End,
 }
 
-pub fn events() -> Subscription<Event> {
+pub fn events() -> Subscription<(window::Id, Event)> {
     event::listen_with(filtered_events)
 }
 
-fn filtered_events(event: iced::Event, status: iced::event::Status) -> Option<Event> {
+fn filtered_events(
+    event: iced::Event,
+    status: iced::event::Status,
+    window: window::Id,
+) -> Option<(window::Id, Event)> {
     let ignored = |status| matches!(status, iced::event::Status::Ignored);
 
-    match &event {
+    let event = match &event {
         iced::Event::Keyboard(keyboard::Event::KeyPressed {
             key: keyboard::Key::Named(keyboard::key::Named::Escape),
             ..
@@ -34,7 +37,8 @@ fn filtered_events(event: iced::Event, status: iced::event::Status) -> Option<Ev
             key: keyboard::Key::Named(keyboard::key::Named::End),
             ..
         }) if ignored(status) => Some(Event::End),
-        iced::Event::Window(_, window::Event::CloseRequested) => Some(Event::CloseRequested),
         _ => None,
-    }
+    };
+
+    event.map(|event| (window, event))
 }
