@@ -1,9 +1,9 @@
 use data::Config;
-use iced::widget::{column, container, text};
+use iced::widget::{column, container, pane_grid, text};
 use iced::Length;
 
-use crate::theme;
 use crate::widget::{combo_box, double_pass, key_press, Element};
+use crate::{theme, window};
 
 #[derive(Debug, Clone)]
 pub struct CommandBar {
@@ -21,15 +21,19 @@ pub enum Message {
 impl CommandBar {
     pub fn new(
         buffers: &[data::Buffer],
+        version: &data::Version,
         config: &Config,
-        is_focused_buffer: bool,
+        focus: Option<(window::Id, pane_grid::Pane)>,
         resize_buffer: data::buffer::Resize,
+        main_window: window::Id,
     ) -> Self {
         let state = combo_box::State::new(Command::list(
             buffers,
             config,
-            is_focused_buffer,
+            focus,
             resize_buffer,
+            version,
+            main_window,
         ));
         state.focus();
 
@@ -51,9 +55,11 @@ impl CommandBar {
     pub fn view<'a>(
         &'a self,
         buffers: &[data::Buffer],
-        focused_buffer: bool,
+        focus: Option<(window::Id, pane_grid::Pane)>,
         resize_buffer: data::buffer::Resize,
+        version: &data::Version,
         config: &'a Config,
+        main_window: window::Id,
     ) -> Element<'a, Message> {
         // 1px larger than default
         let font_size = config.font.size.map(f32::from).unwrap_or(theme::TEXT_SIZE) + 1.0;
@@ -78,9 +84,9 @@ impl CommandBar {
             column(
                 std::iter::once(text("Type a command...").size(font_size))
                     .chain(
-                        Command::list(buffers, config, focused_buffer, resize_buffer)
+                        Command::list(buffers, config, focus, resize_buffer, version, main_window)
                             .iter()
-                            .map(|command| text(command).size(font_size)),
+                            .map(|command| text(command.to_string()).size(font_size)),
                     )
                     .map(Element::from),
             )
@@ -88,7 +94,7 @@ impl CommandBar {
             .padding([0, 20]),
             container(combo_box)
                 .padding(1)
-                .style(theme::container::context)
+                .style(theme::container::tooltip)
                 .width(Length::Fill),
         )
     }
@@ -102,23 +108,34 @@ pub enum Event {
 
 #[derive(Debug, Clone)]
 pub enum Command {
+    Version(Version),
     Buffer(Buffer),
     Configuration(Configuration),
     UI(Ui),
     Theme(Theme),
 }
 
+#[derive(Debug, Clone)]
+pub enum Version {
+    Application(data::Version),
+}
+
 #[derive(Debug, Clone)]
 pub enum Buffer {
     Maximize(bool),
     New,
     Close,
     Replace(data::Buffer),
+    Popout,
+    Merge,
+    ToggleFileTransfers,
 }
 
 #[derive(Debug, Clone)]
 pub enum Configuration {
-    Open,
+    Reload,
+    OpenDirectory,
+    OpenWebsite,
 }
 
 #[derive(Debug, Clone)]
@@ -129,16 +146,19 @@ pub enum Ui {
 #[derive(Debug, Clone)]
 pub enum Theme {
     Switch(data::Theme),
+    OpenEditor,
 }
 
 impl Command {
     pub fn list(
         buffers: &[data::Buffer],
         config: &Config,
-        is_focused_buffer: bool,
+        focus: Option<(window::Id, pane_grid::Pane)>,
         resize_buffer: data::buffer::Resize,
+        version: &data::Version,
+        main_window: window::Id,
     ) -> Vec<Self> {
-        let buffers = Buffer::list(buffers, is_focused_buffer, resize_buffer)
+        let buffers = Buffer::list(buffers, focus, resize_buffer, main_window)
             .into_iter()
             .map(Command::Buffer);
 
@@ -150,7 +170,14 @@ impl Command {
 
         let themes = Theme::list(config).into_iter().map(Command::Theme);
 
-        buffers.chain(configs).chain(themes).chain(uis).collect()
+        let version = Version::list(version).into_iter().map(Command::Version);
+
+        version
+            .chain(buffers)
+            .chain(configs)
+            .chain(themes)
+            .chain(uis)
+            .collect()
     }
 }
 
@@ -161,6 +188,7 @@ impl std::fmt::Display for Command {
             Command::Configuration(config) => write!(f, "Configuration: {}", config),
             Command::UI(ui) => write!(f, "UI: {}", ui),
             Command::Theme(theme) => write!(f, "Theme: {}", theme),
+            Command::Version(application) => write!(f, "Version: {}", application),
         }
     }
 }
@@ -168,12 +196,13 @@ impl std::fmt::Display for Command {
 impl Buffer {
     fn list(
         buffers: &[data::Buffer],
-        is_focused_buffer: bool,
+        focus: Option<(window::Id, pane_grid::Pane)>,
         resize_buffer: data::buffer::Resize,
+        main_window: window::Id,
     ) -> Vec<Self> {
-        let mut list = vec![Buffer::New];
+        let mut list = vec![Buffer::New, Buffer::ToggleFileTransfers];
 
-        if is_focused_buffer {
+        if let Some((window, _)) = focus {
             list.push(Buffer::Close);
 
             match resize_buffer {
@@ -182,6 +211,12 @@ impl Buffer {
                 data::buffer::Resize::None => {}
             }
 
+            if window == main_window {
+                list.push(Buffer::Popout);
+            } else {
+                list.push(Buffer::Merge);
+            }
+
             list.extend(buffers.iter().cloned().map(Buffer::Replace));
         }
 
@@ -189,9 +224,19 @@ impl Buffer {
     }
 }
 
+impl Version {
+    fn list(version: &data::Version) -> Vec<Self> {
+        vec![Version::Application(version.clone())]
+    }
+}
+
 impl Configuration {
     fn list() -> Vec<Self> {
-        vec![Configuration::Open]
+        vec![
+            Configuration::OpenDirectory,
+            Configuration::OpenWebsite,
+            Configuration::Reload,
+        ]
     }
 }
 
@@ -203,16 +248,30 @@ impl Ui {
 
 impl Theme {
     fn list(config: &Config) -> Vec<Self> {
-        config
-            .themes
-            .all
-            .iter()
-            .cloned()
-            .map(Self::Switch)
+        Some(Self::OpenEditor)
+            .into_iter()
+            .chain(config.themes.all.iter().cloned().map(Self::Switch))
             .collect()
     }
 }
 
+impl std::fmt::Display for Version {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            Version::Application(version) => {
+                let latest = version
+                    .remote
+                    .as_ref()
+                    .filter(|remote| remote != &&version.current)
+                    .map(|remote| format!("(Latest: {})", remote))
+                    .unwrap_or("(Latest release)".to_owned());
+
+                write!(f, "{} {}", version.current, latest)
+            }
+        }
+    }
+}
+
 impl std::fmt::Display for Buffer {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
@@ -236,6 +295,9 @@ impl std::fmt::Display for Buffer {
                 }
                 data::Buffer::Query(_, nick) => write!(f, "Change to {}", nick),
             },
+            Buffer::Popout => write!(f, "Pop out buffer"),
+            Buffer::Merge => write!(f, "Merge buffer"),
+            Buffer::ToggleFileTransfers => write!(f, "Toggle File Transfers"),
         }
     }
 }
@@ -243,7 +305,9 @@ impl std::fmt::Display for Buffer {
 impl std::fmt::Display for Configuration {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            Configuration::Open => write!(f, "Open directory"),
+            Configuration::OpenDirectory => write!(f, "Open config directory"),
+            Configuration::OpenWebsite => write!(f, "Open wiki website"),
+            Configuration::Reload => write!(f, "Reload config file"),
         }
     }
 }
@@ -260,6 +324,7 @@ impl std::fmt::Display for Theme {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
             Theme::Switch(theme) => write!(f, "Switch to {}", theme.name),
+            Theme::OpenEditor => write!(f, "Open editor"),
         }
     }
 }
