@@ -1,12 +1,18 @@
-use data::theme::Colors;
 use iced::application;
 
 use crate::widget::combo_box;
 
+pub use data::theme::{
+    color_to_hex, hex_to_color, Buffer, Button, Buttons, Colors, General, ServerMessages, Text,
+};
+
 pub mod button;
+pub mod checkbox;
 pub mod container;
+pub mod context_menu;
 pub mod menu;
 pub mod pane_grid;
+pub mod progress_bar;
 pub mod rule;
 pub mod scrollable;
 pub mod selectable_text;
@@ -45,7 +51,7 @@ impl Theme {
         }
     }
 
-    fn colors(&self) -> &Colors {
+    pub fn colors(&self) -> &Colors {
         match self {
             Theme::Selected(selected) => &selected.colors,
             Theme::Preview { preview, .. } => &preview.colors,
@@ -68,20 +74,10 @@ impl Default for Theme {
 impl application::DefaultStyle for Theme {
     fn default_style(&self) -> application::Appearance {
         application::Appearance {
-            background_color: self.colors().background.base,
-            text_color: self.colors().text.base,
+            background_color: self.colors().general.background,
+            text_color: self.colors().text.primary,
         }
     }
 }
 
-impl combo_box::DefaultStyle for Theme {
-    fn default_style() -> combo_box::Style<'static, Self> {
-        combo_box::Style {
-            text_input: Box::new(text_input::primary),
-            menu: menu::Style {
-                list: Box::new(menu::combo_box),
-                scrollable: Box::new(scrollable::primary),
-            },
-        }
-    }
-}
+impl combo_box::Catalog for Theme {}
